/* Y o u r   D e s c r i p t i o n                       */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.11  */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "abimport.h"
#include "proto.h"

PtRegionWidget_t *widget_region_open( void );
PtRegionWidget_t *region_widget;
extern char *filename;
extern FILE *logfile;

static char buff[255];
const char *key( unsigned short sym );
int g_mouse = 1, g_mouse_repeat = 1, g_keys = 1, g_key_repeat = 1;

void cput( const char *txt, unsigned value, int flag )
  {
    sprintf(buff, "%s %-14s ", txt, flag ? key(value) : "---" );
	PtTerminalPuts(ABW_Text, buff);
	if (logfile) fputs(buff, logfile);
  }

int keys( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
{
	g_keys = !g_keys;
	return( Pt_CONTINUE );
}
int key_repeat( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
{
	g_key_repeat = !g_key_repeat;
	return( Pt_CONTINUE );
}
int mouse( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
{
	g_mouse = !g_mouse;
	return( Pt_CONTINUE );
}
int mouse_repeat( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
{
	g_mouse_repeat = !g_mouse_repeat;
	return( Pt_CONTINUE );
}

int
open_raw( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
  {
	region_widget = widget_region_open( );

	return( Pt_CONTINUE );
  }



int widget_region_callback( PtWidget_t *widget, void *data, PtCallbackInfo_t *cbinfo )
{
	/* eliminate warnings */
	widget = widget, cbinfo = cbinfo; data = data;


	if( cbinfo->event->type == Ph_EV_RAW )
	  {
	  PhEvent_t *ev = cbinfo->event;
#ifdef DEBUG_PRINTING
		printf("EV RAW - src.rid:%d  subtype: %X\n", ev->emitter.rid, ev->subtype );
#endif
		PtTerminalPuts(ABW_Text, "Got a raw event...\r\n");
		if (logfile) fputs("Got a raw event...\r\n", logfile);

	  }
	else if( cbinfo->event->type == Ph_EV_KEY )
	  {
		PhKeyEvent_t *kev = PhGetData( cbinfo->event );
		if (g_keys) return  Pt_CONTINUE;
		if ((kev->key_flags & 2) && g_key_repeat) return  Pt_CONTINUE;
		
		cput( "cap", kev->key_cap, kev->key_flags & Pk_KF_Cap_Valid );
		cput( "sym", kev->key_sym, kev->key_flags & Pk_KF_Sym_Valid );
		if ( kev->key_flags & Pk_KF_Scan_Valid )
    		sprintf(buff, "scan %02X ", kev->key_scan );
		else
    		sprintf(buff, "scan -- " );
		PtTerminalPuts(ABW_Text, buff);
		if (logfile) fputs(buff, logfile);
		sprintf(buff, "mod %04X flags %04X", kev->key_mods, kev->key_flags );
		PtTerminalPuts(ABW_Text, buff);
		if (logfile) fputs(buff, logfile);
#if 0
        if ( inp->length ) 
		  {
            PtTerminalPuts(ABW_Text, ":" );
			if (logfile) fputs(":", logfile);
            for ( i=0; i<inp->length; ++i ) 
			  {
				unsigned char c = inp->string[i];
                sprintf( buff, c >= ' ' && c < 0x7F ? " '%c'" : " %02X", c );
                PtTerminalPuts(ABW_Text, buff);
				if (logfile) fputs(buff, logfile);
              }
		  }
#endif
		PtTerminalPuts(ABW_Text,"\r\n" );
		if (logfile) fputs("\r\n", logfile);

	  }
	else if( cbinfo->event->type == Ph_EV_PTR_MOTION_NOBUTTON )
	  {
		PhPointerEvent_t const *const ev = PhGetData( cbinfo->event );

		if (g_mouse) return  Pt_CONTINUE;

		sprintf(buff,"m(%d,%d)",ev->pos.x, ev->pos.y);
		PtTerminalPuts(ABW_Text, buff);
		if (logfile) fputs(buff, logfile);
	  }
	else if( cbinfo->event->type == Ph_EV_PTR_MOTION_BUTTON )
	  {
		PhPointerEvent_t const *const ev = PhGetData( cbinfo->event );

		if (g_mouse) return  Pt_CONTINUE;

		PtTerminalPuts(ABW_Text, "M");
		if (logfile) fputs("M", logfile);
	
	  }
	else if( cbinfo->event->type == Ph_EV_BUT_PRESS )
	  {
		PhPointerEvent_t const *const ev = PhGetData( cbinfo->event );
		sprintf(buff,"(%4d,%4d) %1d %04x %1d %04x %6x  Button Press\n\r",
				ev->pos.x, ev->pos.y,
				ev->buttons,
				ev->button_state,
				ev->click_count,
				ev->flags,
				ev->key_mods);
		PtTerminalPuts(ABW_Text, buff);
		if (logfile) fputs(buff, logfile);

	  }
	else if( cbinfo->event->type == Ph_EV_BUT_RELEASE )
	  {
		PhPointerEvent_t const *const ev = PhGetData( cbinfo->event );
		sprintf(buff,"(%4d,%4d) %1d %04x %1d %04x %6x  Button Release\n\r",
				ev->pos.x, ev->pos.y,
				ev->buttons,
				ev->button_state,
				ev->click_count,
				ev->flags,
				ev->key_mods);
		PtTerminalPuts(ABW_Text, buff);
		if (logfile) fputs(buff, logfile);
	  }
	else if( cbinfo->event->type == Ph_EV_BUT_REPEAT )
	  {
		PhPointerEvent_t const *const ev = PhGetData( cbinfo->event );

		if (g_mouse_repeat) return  Pt_CONTINUE;

		sprintf(buff,"(%4d,%4d) %1d %04x %1d %04x %6x  Button Repeat\n\r",
				ev->pos.x, ev->pos.y,
				ev->buttons,
				ev->button_state,
				ev->click_count,
				ev->flags,
				ev->key_mods);
		PtTerminalPuts(ABW_Text, buff);
		if (logfile) fputs(buff, logfile);

	  }
	else if( cbinfo->event->type == Ph_EV_RAW )
	  {
		PtTerminalPuts(ABW_Text, "*** RAW EVENT!! ***\r\n");
		if (logfile) fputs("*** RAW EVENT!! ***\r\n", logfile);
	  }
	else
	  {
		PtTerminalPuts(ABW_Text, "Got an unknown event...\r\n");
		if (logfile) fputs("Got an unknown event...\r\n", logfile);
	  }
	return( Pt_CONTINUE );
}



//
// Create screen-wide force-front region to trap events 
//
PtRegionWidget_t *widget_region_open( void )
{
	PhRect_t 		rect = {{0,0}, {SHRT_MAX, SHRT_MAX}};
	// PhRect_t 		rect = {{-SHRT_MAX, -SHRT_MAX}, {SHRT_MAX, SHRT_MAX}};
	PhRid_t 		parent_rid = 0;
	PhArea_t		area; 
	PtWidget_t 	   *ret_wgt;  // the region widget...
    PtArg_t			args[10];
	int				n = 0;
	PtRawCallback_t	raw_cb;
	
	/* setup region and rect */
	PtSetArg( &args[n], Pt_ARG_REGION_PARENT, parent_rid, 0 );n++;
	PtSetArg( &args[n], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0 );n++;
	
	if ( !parent_rid ) {
		PtSetArg( &args[n], Pt_ARG_REGION_FLAGS, 
							Ph_FORCE_FRONT , 
							Ph_FORCE_FRONT|Ph_FORCE_BOUNDARY ) ; n++; 
	}
	PtSetArg( &args[n], Pt_ARG_REGION_SENSE, Ph_EV_RAW | Ph_EV_KEY | Ph_EV_PTR_ALL, ~0); n++;
	PtSetArg( &args[n], Pt_ARG_REGION_OPAQUE, NULL, ~0);n++;

if (rect.ul.x > rect.lr.x) fprintf(stderr,"rectangle not sorted ul=%d lr=%d, (FILE=%s LINE=%d)\n",rect.ul.x,rect.lr.x,__FILE__,__LINE__);
if (rect.ul.y > rect.lr.y) fprintf(stderr,"rectangle not sorted ul=%d lr=%d, (FILE=%s LINE=%d)\n",rect.ul.y,rect.lr.y,__FILE__,__LINE__);
	area.pos.x =  rect.ul.x;
	area.pos.y =  rect.ul.y;
	area.size.h = rect.lr.x - rect.ul.x;
	area.size.w = rect.lr.y - rect.ul.y;
	PtSetArg( &args[n], Pt_ARG_AREA, &area, 0 ); n++;
	
	raw_cb.event_f = widget_region_callback;
//	raw_cb.data = widget;
	raw_cb.data = NULL;
	raw_cb.event_mask = Ph_EV_RAW  | Ph_EV_KEY | Ph_EV_PTR_ALL;
	PtSetArg( &args[n], Pt_CB_RAW_EVENT, &raw_cb, 0 ); n++;
	PtSetArg( &args[n], Pt_ARG_REGION_FIELDS, Ph_REGION_EV_SENSE |
		   									  Ph_REGION_EV_OPAQUE |
											  Ph_REGION_PARENT |
											  Ph_REGION_FLAGS |
											  Ph_REGION_RECT,
											  //---------------------
											  Ph_REGION_EV_SENSE |
		   									  Ph_REGION_EV_OPAQUE |
		   									  Ph_REGION_PARENT |
											  Ph_REGION_FLAGS |
											  Ph_REGION_RECT  ); n++;

	/* open FF region */
	PtSetParentWidget( NULL );
	if ( ( ret_wgt = PtCreateWidget( PtRegion, NULL, n, args ) ) == NULL )
		return(NULL);
	PtRealizeWidget( ret_wgt );
	return( (PtRegionWidget_t *) ret_wgt );
}

const char *key( unsigned short sym ) 
  {
    static char buf[ 30 ];
    struct Sym {
        unsigned short val;
        const char *name;
        }
        const *s;
    static const struct Sym tab[] = {
        { Pk_BackSpace, "BackSpace" },
        { Pk_Tab, "Tab" },
        { Pk_Linefeed, "Linefeed" },
        { Pk_Clear, "Clear" },
        { Pk_Return, "Return" },
        { Pk_Pause, "Pause" },
        { Pk_Scroll_Lock, "Scroll_Lock" },
        { Pk_Escape, "Escape" },
        { Pk_Delete, "Delete" },
        { Pk_Home, "Home" },
        { Pk_Left, "Left" },
        { Pk_Up, "Up" },
        { Pk_Right, "Right" },
        { Pk_Down, "Down" },
        { Pk_Pg_Up, "Pg_Up" },
        { Pk_Prior, "Prior" },
        { Pk_Pg_Down, "Pg_Down" },
        { Pk_Next, "Next" },
        { Pk_End, "End" },
        { Pk_Begin, "Begin" },
        { Pk_Select, "Select" },
        { Pk_Print, "Print" },
        { Pk_Execute, "Execute" },
        { Pk_Insert, "Insert" },
        { Pk_Undo, "Undo" },
        { Pk_Redo, "Redo" },
        { Pk_Menu, "Menu" },
        { Pk_Find, "Find" },
        { Pk_Cancel, "Cancel" },
        { Pk_Help, "Help" },
        { Pk_Break, "Break" },
        { Pk_Mode_switch, "Mode_switch" },
        { Pk_Num_Lock, "Num_Lock" },
        { Pk_KP_Space, "KP_Space" },
        { Pk_KP_Tab, "KP_Tab" },
        { Pk_KP_Enter, "KP_Enter" },
        { Pk_KP_F1, "KP_F1" },
        { Pk_KP_F2, "KP_F2" },
        { Pk_KP_F3, "KP_F3" },
        { Pk_KP_F4, "KP_F4" },
        { Pk_KP_Equal, "KP_Equal" },
        { Pk_KP_Multiply, "KP_Multiply" },
        { Pk_KP_Add, "KP_Add" },
        { Pk_KP_Separator, "KP_Separator" },
        { Pk_KP_Subtract, "KP_Subtract" },
        { Pk_KP_Decimal, "KP_Decimal" },
        { Pk_KP_Divide, "KP_Divide" },
        { Pk_KP_0, "KP_0" },
        { Pk_KP_1, "KP_1" },
        { Pk_KP_2, "KP_2" },
        { Pk_KP_3, "KP_3" },
        { Pk_KP_4, "KP_4" },
        { Pk_KP_5, "KP_5" },
        { Pk_KP_6, "KP_6" },
        { Pk_KP_7, "KP_7" },
        { Pk_KP_8, "KP_8" },
        { Pk_KP_9, "KP_9" },
        { Pk_F1, "F1" },
        { Pk_F2, "F2" },
        { Pk_F3, "F3" },
        { Pk_F4, "F4" },
        { Pk_F5, "F5" },
        { Pk_F6, "F6" },
        { Pk_F7, "F7" },
        { Pk_F8, "F8" },
        { Pk_F9, "F9" },
        { Pk_F10, "F10" },
        { Pk_F11, "F11" },
        { Pk_L1, "L1" },
        { Pk_F12, "F12" },
        { Pk_L2, "L2" },
        { Pk_F13, "F13" },
        { Pk_L3, "L3" },
        { Pk_F14, "F14" },
        { Pk_L4, "L4" },
        { Pk_F15, "F15" },
        { Pk_L5, "L5" },
        { Pk_F16, "F16" },
        { Pk_L6, "L6" },
        { Pk_F17, "F17" },
        { Pk_L7, "L7" },
        { Pk_F18, "F18" },
        { Pk_L8, "L8" },
        { Pk_F19, "F19" },
        { Pk_L9, "L9" },
        { Pk_F20, "F20" },
        { Pk_L10, "L10" },
        { Pk_F21, "F21" },
        { Pk_R1, "R1" },
        { Pk_F22, "F22" },
        { Pk_R2, "R2" },
        { Pk_F23, "F23" },
        { Pk_R3, "R3" },
        { Pk_F24, "F24" },
        { Pk_R4, "R4" },
        { Pk_F25, "F25" },
        { Pk_R5, "R5" },
        { Pk_F26, "F26" },
        { Pk_R6, "R6" },
        { Pk_F27, "F27" },
        { Pk_R7, "R7" },
        { Pk_F28, "F28" },
        { Pk_R8, "R8" },
        { Pk_F29, "F29" },
        { Pk_R9, "R9" },
        { Pk_F30, "F30" },
        { Pk_R10, "R10" },
        { Pk_F31, "F31" },
        { Pk_R11, "R11" },
        { Pk_F32, "F32" },
        { Pk_R12, "R12" },
        { Pk_F33, "F33" },
        { Pk_R13, "R13" },
        { Pk_F34, "F34" },
        { Pk_R14, "R14" },
        { Pk_F35, "F35" },
        { Pk_R15, "R15" },
        { Pk_Shift_L, "Shift_L" },
        { Pk_Shift_R, "Shift_R" },
        { Pk_Control_L, "Control_L" },
        { Pk_Control_R, "Control_R" },
        { Pk_Caps_Lock, "Caps_Lock" },
        { Pk_Shift_Lock, "Shift_Lock" },
        { Pk_Meta_L, "Meta_L" },
        { Pk_Meta_R, "Meta_R" },
        { Pk_Alt_L, "Alt_L" },
        { Pk_Alt_R, "Alt_R" },
        { Pk_Super_L, "Super_L" },
        { Pk_Super_R, "Super_R" },
        { Pk_Hyper_L, "Hyper_L" },
        { Pk_Hyper_R, "Hyper_R" },
        { 0, 0 }
        };
    sprintf( buf, "%04X", sym );
    if ( sym >= 0x20 && sym < 0x7F )
        sprintf( buf+4, "='%c'", sym );
    else {
        for ( s = tab; s->val != sym; ++s )
            if ( s->val == 0 )
                return buf;
        sprintf( buf+4, "=%s", s->name );
        }
    return buf;
  }
